@@ -29,6 +29,15 @@
 #include "gnutls_handshake.h"
 #include "gnutls_hash_int.h"
 #include "gnutls_cipher_int.h"
+#include "gnutls_priority.h"
+#include "gnutls_algorithms.h"
+#ifdef HAVE_ERRNO_H
+#include <errno.h>
+#endif
+
+#ifndef EAGAIN
+# define EAGAIN EWOULDBLOCK
+#endif
 
 GNUTLS_Version gnutls_get_current_version(GNUTLS_STATE state) {
 GNUTLS_Version ver;
@@ -59,9 +68,9 @@ int gnutls_init(GNUTLS_STATE * state, ConnectionEnd con_end)
 	(*state)->security_parameters.entity = con_end;
 
 /* Set the defaults (only to remind me that they should be allocated ) */
-	(*state)->security_parameters.bulk_cipher_algorithm = GNUTLS_NULL;
-	(*state)->security_parameters.mac_algorithm = GNUTLS_MAC_NULL;
-	(*state)->security_parameters.compression_algorithm = GNUTLS_COMPRESSION_NULL;
+	(*state)->security_parameters.bulk_cipher_algorithm = GNUTLS_NULL_CIPHER;
+	(*state)->security_parameters.mac_algorithm = GNUTLS_NULL_MAC;
+	(*state)->security_parameters.compression_algorithm = GNUTLS_NULL_COMPRESSION;
 
 	(*state)->connection_state.read_compression_state = NULL;
 	(*state)->connection_state.read_mac_secret = NULL;
@@ -105,6 +114,16 @@ int gnutls_init(GNUTLS_STATE * state, ConnectionEnd con_end)
 	(*state)->gnutls_internals.CompressionMethodPriority.algorithm_priority=NULL;
 	(*state)->gnutls_internals.CompressionMethodPriority.algorithms=0;
 
+	/* Set default priorities */
+	gnutls_set_cipher_priority( (*state), 2, GNUTLS_RIJNDAEL, GNUTLS_3DES);
+	gnutls_set_compression_priority( (*state), 1, GNUTLS_NULL_COMPRESSION);
+	gnutls_set_kx_priority( (*state), 2, GNUTLS_KX_DHE_DSS, GNUTLS_KX_DHE_RSA);
+	gnutls_set_mac_priority( (*state), 2, GNUTLS_MAC_SHA, GNUTLS_MAC_MD5);
+
+	(*state)->security_parameters.session_id_size = 0;
+	(*state)->gnutls_internals.resumed_security_parameters.session_id_size = 0;
+	(*state)->gnutls_internals.resumed = RESUME_FALSE;
+
 	return 0;
 }
 
@@ -150,7 +169,7 @@ int gnutls_deinit(GNUTLS_STATE * state)
 	return 0;
 }
 
-
+inline
 static void *_gnutls_cal_PRF_A( MACAlgorithm algorithm, void *secret, int secret_size, void *seed, int seed_size)
 {
 	GNUTLS_MAC_HANDLE td1;
@@ -214,13 +233,32 @@ static svoid *gnutls_P_hash( MACAlgorithm algorithm, opaque * secret, int secret
 	return ret;
 }
 
+/* Function that xor's buffers using the maximum word size supported
+ * by the system. It should be faster.
+ */
+inline static
+void _gnutls_xor(void* _o1, void* _o2, int _length) {
+unsigned long int* o1 = _o1;
+unsigned long int* o2 = _o2;
+int i, length = _length/sizeof(unsigned long int);
+int modlen = _length%sizeof(unsigned long int);
+
+	for (i = 0; i < length; i++) {
+		o1[i] ^= o2[i];
+	}
+	i*=sizeof(unsigned long int);
+	for (;i<modlen;i++) {
+		((char*)_o1)[i] ^= ((char*)_o2)[i];
+	}
+	return ;
+}
 
 /* The PRF function expands a given secret 
  * needed by the TLS specification
  */
 svoid *gnutls_PRF( opaque * secret, int secret_size, uint8 * label, int label_size, opaque * seed, int seed_size, int total_bytes)
 {
-	int l_s, i, s_seed_size;
+	int l_s, s_seed_size;
 	char *o1, *o2;
 	char *s1, *s2;
 	char *s_seed;
@@ -244,9 +282,7 @@ svoid *gnutls_PRF( opaque * secret, int secret_size, uint8 * label, int label_si
 
 	gnutls_free(s_seed);
 
-	for (i = 0; i < total_bytes; i++) {
-		o1[i] ^= o2[i];
-	}
+	_gnutls_xor(o1, o2, total_bytes);
 
 	secure_free(o2);
 
@@ -263,7 +299,7 @@ int _gnutls_set_keys(GNUTLS_STATE state)
 {
 	char *key_block;
 	char keyexp[] = "key expansion";
-	char *random = gnutls_malloc(64);
+	char random[64];
 	int hash_size;
 	int IV_size;
 	int key_size;
@@ -313,7 +349,7 @@ int _gnutls_send_alert(int cd, GNUTLS_STATE state, AlertLevel level, AlertDescri
 	memmove(&data[0], &level, 1);
 	memmove(&data[1], &desc, 1);
 
-	return gnutls_send_int(cd, state, GNUTLS_ALERT, data, 2);
+	return gnutls_send_int(cd, state, GNUTLS_ALERT, data, 2, 0);
 
 }
 
@@ -323,8 +359,8 @@ int gnutls_close(int cd, GNUTLS_STATE state)
 
 	ret = _gnutls_send_alert(cd, state, GNUTLS_WARNING, GNUTLS_CLOSE_NOTIFY);
 
-	/* receive pending data or the closure alert */
-	gnutls_recv_int(cd, state, GNUTLS_ALERT, NULL, 0); 
+	/* receive the closure alert */
+	gnutls_recv_int(cd, state, GNUTLS_ALERT, NULL, 0, 0); 
 
 	state->gnutls_internals.valid_connection = VALID_FALSE;
 
@@ -348,10 +384,10 @@ int gnutls_close_nowait(int cd, GNUTLS_STATE state)
  * It is intended to transfer data, under the current state.    
  */
 #define MAX_ENC_LEN 16384
-ssize_t gnutls_send_int(int cd, GNUTLS_STATE state, ContentType type, void *_data, size_t sizeofdata)
+ssize_t gnutls_send_int(int cd, GNUTLS_STATE state, ContentType type, void *_data, size_t sizeofdata, int flags)
 {
 	uint8 *cipher;
-	int i, err, cipher_size;
+	int i, cipher_size;
 	int ret = 0;
 	int iterations;
 	uint16 length;
@@ -439,7 +475,7 @@ ssize_t gnutls_send_int(int cd, GNUTLS_STATE state, ContentType type, void *_dat
 ssize_t _gnutls_send_change_cipher_spec(int cd, GNUTLS_STATE state)
 {
 	uint16 length;
-	int ret = 0, Size;
+	int ret = 0;
 	uint8 type=GNUTLS_CHANGE_CIPHER_SPEC;
 	char data[1] = { GNUTLS_TYPE_CHANGE_CIPHER_SPEC };
 	uint8 headers[5];
@@ -452,7 +488,7 @@ ssize_t _gnutls_send_change_cipher_spec(int cd, GNUTLS_STATE state)
 	headers[1] = state->connection_state.version.major;
 	headers[2] = state->connection_state.version.minor;
 
-#ifdef HARD_DEBUG
+#ifdef HANDSHAKE_DEBUG
 	fprintf(stderr, "Send Change Cipher Spec\n");
 #endif
 
@@ -481,17 +517,32 @@ ssize_t _gnutls_send_change_cipher_spec(int cd, GNUTLS_STATE state)
 	return ret;
 }
 
+#define RCVLOWAT 1 /* this is the default for TCP - just don't change that! */
+
+static int _gnutls_clear_peeked_data( int cd, GNUTLS_STATE state) {
+char peekdata;
+
+	/* this was already read by using MSG_PEEK - so it shouldn't fail */
+	_gnutls_Read( cd, &peekdata, RCVLOWAT, 0); 
+
+	return 0;
+}
+
 /* This function behave exactly like read(). The only difference is 
  * that it accepts, the gnutls_state and the ContentType of data to
- * send (if called by the user the Content is specific)
+ * send (if called by the user the Content is Userdata only)
  * It is intended to receive data, under the current state.
+ * flags is the sockets flags to use. Currently only MSG_DONTWAIT is
+ * supported.
  */
+#define HEADER_SIZE 5
 #define MAX_RECV_SIZE 18432 	/* 2^14+2048 */
-ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data, size_t sizeofdata)
+ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data, size_t sizeofdata, int flags)
 {
 	uint8 *tmpdata;
 	int tmplen;
 	GNUTLS_Version version;
+	uint8 headers[HEADER_SIZE];
 	uint8 recv_type;
 	uint16 length;
 	uint8 *ciphertext;
@@ -502,6 +553,13 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 	 */
 	if ( (type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE) && gnutls_getDataBufferSize(type, state) > 0) {
 		ret = gnutls_getDataFromBuffer(type, state, data, sizeofdata);
+
+		if (type==GNUTLS_APPLICATION_DATA) {
+			/* if the buffer just got empty */
+			if (gnutls_getDataBufferSize(type, state)==0) {
+				_gnutls_clear_peeked_data( cd, state);
+			}
+		}
 		return ret;
 	}
 
@@ -509,28 +567,27 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 		return GNUTLS_E_INVALID_SESSION;
 	}
 
-	if ( _gnutls_Read(cd, &recv_type, 1) != 1) {
+	/* in order for GNUTLS_E_AGAIN to be returned the socket
+	 * must be set to non blocking mode
+	 */
+	if ( _gnutls_Read(cd, headers, HEADER_SIZE, MSG_PEEK|flags) != HEADER_SIZE) {
+		if (errno==EAGAIN) return GNUTLS_E_AGAIN;
 		state->gnutls_internals.valid_connection = VALID_FALSE;
+		if (type==GNUTLS_ALERT) return 0; /* we were expecting close notify */
 		state->gnutls_internals.resumable = RESUME_FALSE;
 		gnutls_assert();
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 	}
 
+	memcpy( &recv_type, &headers[0], 1);
+	memcpy( &version.major, &headers[1], 1);
+	memcpy( &version.minor, &headers[2], 1);
 	version.local = 0; /* TLS/SSL 3.0 */
-	
-	if (_gnutls_Read(cd, &version.major, 1) != 1) {
-		state->gnutls_internals.valid_connection = VALID_FALSE;
-		state->gnutls_internals.resumable = RESUME_FALSE;
-		gnutls_assert();
-		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
-	}
 
-	if (_gnutls_Read(cd, &version.minor, 1) != 1) {
-		state->gnutls_internals.valid_connection = VALID_FALSE;
-		state->gnutls_internals.resumable = RESUME_FALSE;
-		gnutls_assert();
-		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
-	}
+	memcpy( &length, &headers[3], 2);
+#ifndef WORDS_BIGENDIAN
+	length = byteswap16(length);
+#endif
 
 	if (_gnutls_version_is_supported(state, version) == 0) {
 #ifdef DEBUG
@@ -544,15 +601,6 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 		gnutls_set_current_version(state, version);
 	}
 
-	if (_gnutls_Read(cd, &length, 2) != 2) {
-		state->gnutls_internals.valid_connection = VALID_FALSE;
-		state->gnutls_internals.resumable = RESUME_FALSE;
-		gnutls_assert();
-		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
-	}
-#ifndef WORDS_BIGENDIAN
-	length = byteswap16(length);
-#endif
 
 #ifdef HARD_DEBUG
 	fprintf(stderr, "Expected Packet[%d] %s(%d) with length: %d\n",
@@ -572,12 +620,39 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 	}
 
-	ciphertext = gnutls_malloc(length);
+	ciphertext = gnutls_malloc(length+HEADER_SIZE);
 
-	/* read ciphertext */
+/* check if we have that data into buffer. This seems to be
+ * expensive - but this is the only way to handle Non Blocking IO.
+ */
+	if ( _gnutls_Read(cd, ciphertext, HEADER_SIZE+length, MSG_PEEK|flags) != length+HEADER_SIZE) {
+		gnutls_free(ciphertext);
+		
+		if (errno==EAGAIN) return GNUTLS_E_AGAIN;
+		state->gnutls_internals.valid_connection = VALID_FALSE;
+		state->gnutls_internals.resumable = RESUME_FALSE;
+		gnutls_assert();
+		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;		
+	}
+/* ok now we are sure that we can read all the data - so
+ * move on !
+ */
+	_gnutls_Read(cd, headers, HEADER_SIZE, 0); /* read and clear the headers - again! */
 
-	ret = _gnutls_Read(cd, ciphertext, length);
+/* Read the whole packet - again? */	
+	if ( type==GNUTLS_APPLICATION_DATA) {
+		/* get the data - but do not free the buffer in the kernel */
+		ret = _gnutls_Read(cd, ciphertext, length-RCVLOWAT, 0);
+		if (ret>=0)
+			ret += _gnutls_Read(cd, &ciphertext[length-RCVLOWAT], RCVLOWAT, MSG_PEEK);
 
+	} else { /* our - internal data */
+		ret = _gnutls_Read(cd, ciphertext, length, 0);
+	}
+
+	/* Oooops... very rare case since we know that the system HAD 
+	 * received that data.
+	 */
 	if (ret != length) {
 #ifdef DEBUG
 		fprintf(stderr, "Received packet with length: %d\nExpected %d\n", ret, length);
@@ -601,7 +676,7 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 		return 0;
 	}
 
-	tmplen = _gnutls_decrypt( state, ciphertext, length, &tmpdata, type);
+	tmplen = _gnutls_decrypt( state, ciphertext, length, &tmpdata, recv_type);
 	if (tmplen < 0) {
 		switch (tmplen) {
 			case GNUTLS_E_MAC_FAILED:
@@ -628,7 +703,7 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 	} else {
 		switch (recv_type) {
 		case GNUTLS_ALERT:
-#ifdef HARD_DEBUG
+#ifdef DEBUG
 			fprintf(stderr, "Alert[%d|%d] - %s - was received\n", tmpdata[0], tmpdata[1], _gnutls_alert2str((int)tmpdata[1]));
 #endif
 			state->gnutls_internals.last_alert = tmpdata[1];
@@ -657,8 +732,14 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 			/* this packet is now handled above */
 			gnutls_assert();
 			return GNUTLS_E_UNEXPECTED_PACKET;
-
+		case GNUTLS_APPLICATION_DATA:
+			/* even if data is unexpected put it into the buffer */
+			gnutls_insertDataBuffer(recv_type, state, (void *) tmpdata, tmplen);
+			break;
 		default:
+#ifdef DEBUG
+			fprintf(stderr, "Received Unknown packet %d expecting %d\n", recv_type, type);
+#endif
 			gnutls_assert();
 			return GNUTLS_E_UNKNOWN_ERROR;
 		}
@@ -666,24 +747,39 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 
 
 
-	/* Incread sequence number */
+	/* Increase sequence number */
 	state->connection_state.read_sequence_number++;
 
 
-
 	/* Get Application data from buffer */
 	if ((type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE) && (recv_type == type)) {
 		ret = gnutls_getDataFromBuffer(type, state, data, sizeofdata);
+		if (type==GNUTLS_APPLICATION_DATA) {
+			/* if the buffer just got empty */
+			if (gnutls_getDataBufferSize(type, state)==0) {
+				_gnutls_clear_peeked_data( cd, state);
+			}
+
+		}
 		gnutls_free(tmpdata);
 	} else {
-		if (recv_type != type) {
+		if (recv_type != GNUTLS_APPLICATION_DATA) {
 			gnutls_assert();
 			return GNUTLS_E_RECEIVED_BAD_MESSAGE;
+		} else {
+			ret = 0; /* ok */
 		}
-		gnutls_assert(); /* this shouldn't have happened */
-		ret = GNUTLS_E_RECEIVED_BAD_MESSAGE;
 	}
 
 	return ret;
 }
 
+BulkCipherAlgorithm gnutls_get_current_cipher( GNUTLS_STATE state) {
+	return state->security_parameters.bulk_cipher_algorithm;
+}
+MACAlgorithm gnutls_get_current_mac_algorithm( GNUTLS_STATE state) {
+	return state->security_parameters.mac_algorithm;
+}
+CompressionMethod gnutls_get_current_compression_method( GNUTLS_STATE state) {
+	return state->security_parameters.compression_algorithm;
+}
