@@ -26,6 +26,8 @@
 #include "gnutls_algorithms.h"
 #include "gnutls_hash_int.h"
 #include "gnutls_cipher_int.h"
+#include "gnutls_plaintext.h"
+#include "debug.h"
 
 int _gnutls_encrypt(GNUTLS_STATE state, char *data, size_t data_size,
 		    uint8 ** ciphertext, ContentType type)
@@ -33,7 +35,7 @@ int _gnutls_encrypt(GNUTLS_STATE state, char *data, size_t data_size,
 	GNUTLSPlaintext *gtxt;
 	GNUTLSCompressed *gcomp;
 	GNUTLSCiphertext *gcipher;
-	int total_length = 0, err, i;
+	int total_length = 0, err;
 
 	if (data_size == 0)
 		return 0;
@@ -81,9 +83,7 @@ int _gnutls_decrypt(GNUTLS_STATE state, char *ciphertext,
 	GNUTLSPlaintext *gtxt;
 	GNUTLSCompressed *gcomp;
 	GNUTLSCiphertext gcipher;
-	int iterations, i;
-	int err, ret;
-	int total_length = 0;
+	int ret;
 
 	if (ciphertext_size == 0)
 		return 0;
@@ -139,8 +139,7 @@ int _gnutls_set_cipher(GNUTLS_STATE state, BulkCipherAlgorithm algo)
 			state->security_parameters.cipher_type =
 			    CIPHER_STREAM;
 		}
-		state->security_parameters.is_exportable =
-		    EXPORTABLE_FALSE;
+
 		state->security_parameters.key_material_length =
 		    state->security_parameters.key_size =
 		    _gnutls_cipher_get_key_size(algo);
@@ -204,27 +203,34 @@ int _gnutls_connection_state_init(GNUTLS_STATE state)
 	state->connection_state.write_sequence_number = 0;
 	state->connection_state.read_sequence_number = 0;
 
-/* Update internals from CipherSuite selected */
-
-	rc =
-	    _gnutls_set_cipher(state,
+/* Update internals from CipherSuite selected.
+ * If we are resuming just copy the connection state
+ */
+ 	if (state->gnutls_internals.resumed==RESUME_FALSE) {
+		rc =
+		    _gnutls_set_cipher(state,
 			       _gnutls_cipher_suite_get_cipher_algo
 			       (state->gnutls_internals.current_cipher_suite));
-	if (rc < 0)
-		return rc;
-	rc =
-	    _gnutls_set_mac(state,
+		if (rc < 0)
+			return rc;
+		rc =
+		    _gnutls_set_mac(state,
 			    _gnutls_cipher_suite_get_mac_algo
 			    (state->gnutls_internals.current_cipher_suite));
-	if (rc < 0)
-		return rc;
-
-	rc =
-	    _gnutls_set_compression(state,
-			    state->gnutls_internals.compression_method);
-	if (rc < 0)
-		return rc;
+		if (rc < 0)
+			return rc;
 
+		rc =
+		    _gnutls_set_compression(state,
+				    state->gnutls_internals.compression_method);
+		if (rc < 0)
+			return rc;
+	} else {
+		 memcpy( &state->security_parameters, &state->gnutls_internals.resumed_security_parameters, sizeof(SecurityParameters));
+#ifdef HARD_DEBUG
+		 fprintf(stderr, "Master Secret: %s\n", _gnutls_bin2hex(state->security_parameters.master_secret, 48));
+#endif
+	}
 /* Setup the keys since we have the master secret 
  */
 	_gnutls_set_keys(state);
@@ -243,8 +249,10 @@ int _gnutls_connection_state_init(GNUTLS_STATE state)
 	fprintf(stderr, "Compression: %s\n", _gnutls_compression_get_name(state->security_parameters.compression_algorithm));
 #endif
 
-	gnutls_free(state->connection_state.write_mac_secret);
-	gnutls_free(state->connection_state.read_mac_secret);
+	if (state->connection_state.write_mac_secret!=NULL)
+		gnutls_free(state->connection_state.write_mac_secret);
+	if (state->connection_state.read_mac_secret!=NULL)
+		gnutls_free(state->connection_state.read_mac_secret);
 
 	if (state->connection_state.read_cipher_state != NULL)
 		gnutls_cipher_deinit(state->connection_state.
@@ -254,8 +262,10 @@ int _gnutls_connection_state_init(GNUTLS_STATE state)
 		gnutls_cipher_deinit(state->connection_state.
 				     write_cipher_state);
 
-	gnutls_free(state->connection_state.read_compression_state);
-	gnutls_free(state->connection_state.write_compression_state);
+	if (state->connection_state.read_compression_state!=NULL)
+		gnutls_free(state->connection_state.read_compression_state);
+	if (state->connection_state.write_compression_state!=NULL)
+		gnutls_free(state->connection_state.write_compression_state);
 
 	if (_gnutls_compression_is_ok(state->security_parameters.compression_algorithm) == 0) {
 		state->connection_state.read_compression_state = NULL;
@@ -265,8 +275,7 @@ int _gnutls_connection_state_init(GNUTLS_STATE state)
 		return GNUTLS_E_UNKNOWN_COMPRESSION_ALGORITHM;
 	}
 
-	if (_gnutls_mac_is_ok(state->security_parameters.mac_algorithm) ==
-	    0) {
+	if (_gnutls_mac_is_ok(state->security_parameters.mac_algorithm) == 0) {
 
 		mac_size =
 		    _gnutls_mac_get_digest_size(state->security_parameters.
@@ -300,7 +309,7 @@ int _gnutls_connection_state_init(GNUTLS_STATE state)
 		if (state->connection_state.write_cipher_state ==
 		    GNUTLS_CIPHER_FAILED
 		    && state->security_parameters.bulk_cipher_algorithm !=
-		    GNUTLS_NULL) {
+		    GNUTLS_NULL_CIPHER) {
 			gnutls_assert();
 			return GNUTLS_E_UNKNOWN_CIPHER;
 		}
@@ -327,7 +336,7 @@ int _gnutls_connection_state_init(GNUTLS_STATE state)
 		if (state->connection_state.read_cipher_state ==
 		    GNUTLS_CIPHER_FAILED
 		    && state->security_parameters.bulk_cipher_algorithm !=
-		    GNUTLS_NULL) {
+		    GNUTLS_NULL_CIPHER) {
 			gnutls_assert();
 			return GNUTLS_E_UNKNOWN_CIPHER;
 		}
@@ -346,7 +355,7 @@ int _gnutls_connection_state_init(GNUTLS_STATE state)
 		if (state->connection_state.read_cipher_state ==
 		    GNUTLS_CIPHER_FAILED
 		    && state->security_parameters.bulk_cipher_algorithm !=
-		    GNUTLS_NULL) {
+		    GNUTLS_NULL_CIPHER) {
 			gnutls_assert();
 			return GNUTLS_E_UNKNOWN_CIPHER;
 		}
@@ -373,7 +382,7 @@ int _gnutls_connection_state_init(GNUTLS_STATE state)
 		if (state->connection_state.write_cipher_state ==
 		    GNUTLS_CIPHER_FAILED
 		    && state->security_parameters.bulk_cipher_algorithm !=
-		    GNUTLS_NULL) {
+		    GNUTLS_NULL_CIPHER) {
 			gnutls_assert();
 			return GNUTLS_E_UNKNOWN_CIPHER;
 		}
@@ -430,7 +439,7 @@ int _gnutls_TLSCompressed2TLSCiphertext(GNUTLS_STATE state,
 				     mac_secret_size);
 	}
 	if (td == GNUTLS_MAC_FAILED
-	    && state->security_parameters.mac_algorithm != GNUTLS_MAC_NULL) {
+	    && state->security_parameters.mac_algorithm != GNUTLS_NULL_MAC) {
 		gnutls_free(*cipher);
 		gnutls_free(content);
 		gnutls_assert();
@@ -501,7 +510,6 @@ int _gnutls_TLSCompressed2TLSCiphertext(GNUTLS_STATE state,
 		memset(&data[length - pad], pad - 1, pad);
 		memmove(data, content, compressed->length);
 
-
 		memmove(&data[compressed->length], MAC,
 			state->security_parameters.hash_size);
 		gnutls_cipher_encrypt(state->connection_state.
@@ -571,7 +579,7 @@ int _gnutls_TLSCiphertext2TLSCompressed(GNUTLS_STATE state,
 				     mac_secret_size);
 	}
 	if (td == GNUTLS_MAC_FAILED
-	    && state->security_parameters.mac_algorithm != GNUTLS_MAC_NULL) {
+	    && state->security_parameters.mac_algorithm != GNUTLS_NULL_MAC) {
 		gnutls_free(*compress);
 		gnutls_free(content);
 		gnutls_assert();
@@ -616,7 +624,6 @@ int _gnutls_TLSCiphertext2TLSCompressed(GNUTLS_STATE state,
 		}
 		data = gnutls_malloc(length);
 		memmove(data, content, length);
-
 		compressed->fragment = data;
 		compressed->length = length;
 		compressed->type = ciphertext->type;
