@@ -1,4 +1,7 @@
 int _gnutls_encrypt( GNUTLS_STATE state, char* data, size_t data_size, uint8** ciphertext, ContentType type);
+int _gnutls_decrypt(GNUTLS_STATE state, char *ciphertext,
+		    size_t ciphertext_size, uint8 ** data,
+		    ContentType type);
 int _gnutls_TLSCompressed2TLSCiphertext(GNUTLS_STATE state,
 						      GNUTLSCiphertext**
 						      cipher,
