@@ -39,6 +39,8 @@
 #define ERR(x, y)
 #endif
 
+#define TRUE 1
+#define FALSE 0
 
 /* Calculate The SSL3 Finished message */
 #define SSL3_CLIENT_MSG "CLNT"
@@ -161,7 +163,6 @@ int _gnutls_recv_finished(int cd, GNUTLS_STATE state)
 {
 	uint8 *data, *vrfy;
 	int data_size;
-	uint8* concat;	/* md5+sha1 */
 	int ret;
 	int vrfysize;
 
@@ -248,11 +249,9 @@ static int SelectCompMethod(GNUTLS_STATE state, CompressionMethod * ret, opaque
 
 	x = _gnutls_supported_compression_methods(state, &ciphers);
 	memset(ret, '\0', sizeof(CompressionMethod));
-fprintf(stderr, "datalen: %d\n", datalen);
+
 	for (j = 0; j < datalen; j++) {
 		for (i = 0; i < x; i++) {
-			fprintf(stderr, "cipher[%d] = %d\n", i, (int)ciphers[i]);
-			fprintf(stderr, "data[%d] = %d\n", j, (int)data[j]);
 			if ( ciphers[i] == data[j]) {
 				*ret = ciphers[i];
 				gnutls_free(ciphers);
@@ -300,7 +299,7 @@ int _gnutls_send_handshake(int cd, GNUTLS_STATE state, void *i_data,
 		memmove(&data[pos], i_data, i_datasize - 4);
 
 #ifdef HANDSHAKE_DEBUG
-	fprintf(stderr, "Send HANDSHAKE[%d] of %d bytes\n", type, i_datasize+4);
+	fprintf(stderr, "Send HANDSHAKE[%d] of %ld bytes\n", type, i_datasize+4);
 #endif
 
 	/* Here we keep the handshake messages in order to hash them later!
@@ -370,7 +369,7 @@ int _gnutls_recv_handshake(int cd, GNUTLS_STATE state, uint8 **data,
 	length32 = byteswap32(length32);
 #endif
 #ifdef HANDSHAKE_DEBUG
-	fprintf(stderr, "Received HANDSHAKE[%d] of %d bytes\n", dataptr[0], length32+4);
+	fprintf(stderr, "Received HANDSHAKE[%d] of %ld bytes\n", dataptr[0], length32+4);
 #endif
 
 	dataptr = gnutls_realloc( dataptr, length32+HANDSHAKE_HEADERS_SIZE);
@@ -402,7 +401,7 @@ int _gnutls_recv_handshake(int cd, GNUTLS_STATE state, uint8 **data,
 	case GNUTLS_CLIENT_HELLO:
 	case GNUTLS_SERVER_HELLO:
 		ret = _gnutls_recv_hello(cd, state, &dataptr[HANDSHAKE_HEADERS_SIZE],
-					       length32, NULL, 0);
+					       length32);
 		break;
 	case GNUTLS_CERTIFICATE:
 		ret = _gnutls_recv_certificate(cd, state, &dataptr[HANDSHAKE_HEADERS_SIZE],
@@ -507,10 +506,8 @@ int _gnutls_send_hello(int cd, GNUTLS_STATE state, opaque * SessionID,
 		memmove(&state->security_parameters.client_random[4], rand,
 			28);
 
-		memmove(&data[pos], &cur_time, 4);
-		pos += 4;
-		memmove(&data[pos], rand, 28);
-		pos += 28;
+		memmove(&data[pos], state->security_parameters.client_random, 32);
+		pos += 32;
 
 		memmove(&data[pos++], &session_id_len, 1);
 
@@ -519,7 +516,7 @@ int _gnutls_send_hello(int cd, GNUTLS_STATE state, opaque * SessionID,
 		}
 		pos += session_id_len;
 
-		x = _gnutls_supported_ciphersuites(state, &cipher_suites);
+		x = _gnutls_supported_ciphersuites_sorted(state, &cipher_suites);
 		x *= sizeof(uint16); /* in order to get bytes */
 #ifdef WORDS_BIGENDIAN
 		memmove(&data[pos], &x, sizeof(uint16));
@@ -550,7 +547,6 @@ int _gnutls_send_hello(int cd, GNUTLS_STATE state, opaque * SessionID,
 			memmove(&data[pos++], &compression_methods[i], 1);
 		}
 
-		gcry_free(rand);
 		gnutls_free(cipher_suites);
 		gnutls_free(compression_methods);
 
@@ -599,7 +595,6 @@ int _gnutls_send_hello(int cd, GNUTLS_STATE state, opaque * SessionID,
 		memmove(&data[pos++],
 			&state->gnutls_internals.compression_method, 1);
 
-		gcry_free(rand);
 		ret =
 		    _gnutls_send_handshake(cd, state, data, datalen,
 					   GNUTLS_SERVER_HELLO);
@@ -607,6 +602,8 @@ int _gnutls_send_hello(int cd, GNUTLS_STATE state, opaque * SessionID,
 
 	}
 
+	gcry_free(rand);
+
 	return ret;
 }
 
@@ -615,8 +612,7 @@ int _gnutls_send_hello(int cd, GNUTLS_STATE state, opaque * SessionID,
  * hello message is expected. It uses the gnutls_internals.current_cipher_suite
  * and gnutls_internals.compression_method.
  */
-int _gnutls_recv_hello(int cd, GNUTLS_STATE state, char *data, int datalen,
-		       opaque ** SessionID, int SessionIDnum)
+int _gnutls_recv_hello(int cd, GNUTLS_STATE state, char *data, int datalen)
 {
 	uint8 session_id_len = 0, z;
 	int pos = 0;
@@ -661,11 +657,26 @@ int _gnutls_recv_hello(int cd, GNUTLS_STATE state, char *data, int datalen,
 		fprintf(stderr, "SessionID: %s\n",
 			_gnutls_bin2hex(&data[pos], session_id_len));
 #endif
-		pos += session_id_len;
+		if ( (state->gnutls_internals.resumed_security_parameters.session_id_size > 0) 
+			&& memcmp(&data[pos], state->gnutls_internals.resumed_security_parameters.session_id, session_id_len)==0) {
+			/* resume session */
+			
+#ifdef DEBUG
+			fprintf(stderr, "Resuming session\n");
+#endif
+			memcpy( state->gnutls_internals.resumed_security_parameters.server_random, state->security_parameters.server_random, 32);
+			memcpy( state->gnutls_internals.resumed_security_parameters.client_random, state->security_parameters.client_random, 32);
 
-		/* We should resume an old connection here. This is not
-		 * implemented yet.
-		 */
+			state->gnutls_internals.resumed=RESUME_TRUE; /* we are resuming */
+			return 0;
+		} else {
+			/* keep the session id */
+			state->gnutls_internals.resumed=RESUME_FALSE; /* we are not resuming */
+			state->security_parameters.session_id_size = session_id_len;
+			memcpy( state->security_parameters.session_id, &data[pos], session_id_len);
+		}
+
+		pos += session_id_len;
 
 		memmove(&cipher_suite.CipherSuite, &data[pos], 2);
 		pos += 2;
@@ -673,15 +684,15 @@ int _gnutls_recv_hello(int cd, GNUTLS_STATE state, char *data, int datalen,
 		z = 1;
 		x = _gnutls_supported_ciphersuites(state, &cipher_suites);
 		for (i = 0; i < x; i++) {
-			if (memcmp
-			    (&cipher_suites[i], cipher_suite.CipherSuite,
-			     2) == 0) {
+			if (memcmp (&cipher_suites[i], cipher_suite.CipherSuite, 2) == 0) {
 				z = 0;
-
 			}
 		}
-		if (z != 0)
+		if (z != 0) {
+			gnutls_assert();
 			return GNUTLS_E_UNKNOWN_CIPHER_TYPE;
+		}
+			
 		memmove(state->gnutls_internals.
 			current_cipher_suite.CipherSuite,
 			cipher_suite.CipherSuite, 2);
@@ -773,11 +784,9 @@ int _gnutls_recv_hello(int cd, GNUTLS_STATE state, char *data, int datalen,
 
 int _gnutls_recv_certificate(int cd, GNUTLS_STATE state, char *data, int datalen)
 {
-	uint8 session_id_len = 0, z;
 	int pos = 0;
 	char* certificate_list;
-	int i, ret=0;
-	uint16 x;
+	int ret=0;
 	uint32 sizeOfCert;
 	uint24 num;
 	
@@ -825,15 +834,32 @@ int _gnutls_recv_certificate(int cd, GNUTLS_STATE state, char *data, int datalen
 /* This is the main function in the handshake protocol. This does actually
  * everything. (exchange hello messages etc).
  */
+int gnutls_handshake(int cd, GNUTLS_STATE state) {
+	int ret;
+	
+	ret = gnutls_handshake_begin( cd, state);
+	/* FIXME: check certificate */
+	if (ret==0) ret = gnutls_handshake_finish( cd, state);
+	
+	return ret;
+}
 
-int gnutls_handshake(int cd, GNUTLS_STATE state)
+/* in this function we initiate the handshake and we receive - 
+ * if requested - the certificate. This certificate should be checked
+ * somehow.
+ */
+int gnutls_handshake_begin(int cd, GNUTLS_STATE state)
 {
 	int ret;
 	char *session_id;
 	uint8 session_id_size;
 
 	if (state->security_parameters.entity == GNUTLS_CLIENT) {
-		ret = _gnutls_send_hello(cd, state, NULL, 0);
+#ifdef HARD_DEBUG
+		if (state->gnutls_internals.resumed_security_parameters.session_id_size>0) 
+		fprintf(stderr, "Ask to resume: %s\n", _gnutls_bin2hex(state->gnutls_internals.resumed_security_parameters.session_id, state->gnutls_internals.resumed_security_parameters.session_id_size));
+#endif
+		ret = _gnutls_send_hello(cd, state, state->gnutls_internals.resumed_security_parameters.session_id, state->gnutls_internals.resumed_security_parameters.session_id_size);
 		if (ret < 0) {
 			ERR("send hello", ret);
 			gnutls_clearHashDataBuffer( state);
@@ -849,7 +875,8 @@ int gnutls_handshake(int cd, GNUTLS_STATE state)
 			return ret;
 		}
 
-		/* RECV CERTIFICATE + KEYEXCHANGE + CERTIFICATE_REQUEST */
+		/* RECV CERTIFICATE */
+		if (state->gnutls_internals.resumed==RESUME_FALSE) /* if we are not resuming */
 		ret =
 		    _gnutls_recv_handshake(cd, state, NULL, NULL, GNUTLS_CERTIFICATE);
 		if (ret < 0) {
@@ -857,92 +884,7 @@ int gnutls_handshake(int cd, GNUTLS_STATE state)
 			gnutls_clearHashDataBuffer( state);
 			return ret;
 		}
-		
-
-		/* receive the server key exchange */
-		ret = _gnutls_recv_server_kx_message(cd, state);
-		if (ret < 0) {
-			ERR("recv server kx message", ret);
-			gnutls_clearHashDataBuffer( state);
-			return ret;
-		}
-
-
-		/* receive the server hello done */
-		ret =
-		    _gnutls_recv_handshake(cd, state, NULL, NULL,
-					   GNUTLS_SERVER_HELLO_DONE);
-		if (ret < 0) {
-			ERR("recv server hello done", ret);
-			gnutls_clearHashDataBuffer( state);
-			return ret;
-		}
-
-		/* SEND CERTIFICATE + KEYEXCHANGE + CERTIFICATE_VERIFY */
-		ret = _gnutls_send_client_certificate(cd, state);
-		if (ret < 0) {
-			ERR("send client certificate", ret);
-			gnutls_clearHashDataBuffer( state);
-			return ret;
-		}
-
-
-		ret = _gnutls_send_client_kx_message(cd, state);
-		if (ret < 0) {
-			ERR("send client kx", ret);
-			gnutls_clearHashDataBuffer( state);
-			return ret;
-		}
-
-		ret = _gnutls_send_client_certificate_verify( cd, state);
-		if (ret < 0) {
-			ERR("send client certificate verify", ret);
-			gnutls_clearHashDataBuffer( state);
-			return ret;
-		}
-
-		/* Send the CHANGE CIPHER SPEC PACKET */
-		ret = _gnutls_send_change_cipher_spec(cd, state);
-		if (ret < 0) {
-			ERR("send ChangeCipherSpec", ret);
-			gnutls_clearHashDataBuffer( state);
-			return ret;
-		}
-
-
-		/* Initialize the connection state (start encryption) */
-		ret = _gnutls_connection_state_init(state);
-		if (ret<0) {
-			gnutls_clearHashDataBuffer( state);
-			return ret;
-		}
-
-		/* send the finished message */
-
-		ret = _gnutls_send_finished(cd, state);
-		if (ret < 0) {
-			ERR("send Finished", ret);
-			gnutls_clearHashDataBuffer( state);
-			return ret;
-		}
-
-
-		ret =
-		    gnutls_recv_int(cd, state, GNUTLS_CHANGE_CIPHER_SPEC,
-				    NULL, 0);
-		if (ret < 0) {
-			ERR("recv ChangeCipherSpec", ret);
-			gnutls_clearHashDataBuffer( state);
-			return ret;
-		}
-
-		ret = _gnutls_recv_finished(cd, state);
-		if (ret < 0) {
-			ERR("recv finished", ret);
-			gnutls_clearHashDataBuffer( state);
-			return ret;
-		}
-
+		return 0;
 	} else {		/* SERVER */
 
 		ret =
@@ -965,6 +907,8 @@ int gnutls_handshake(int cd, GNUTLS_STATE state)
 		}
 		gnutls_free(session_id);
 
+		/* FIXME: send our certificate - if required */
+
 		/* SEND CERTIFICATE + KEYEXCHANGE + CERTIFICATE_REQUEST */
 		ret = _gnutls_send_server_kx_message(cd, state);
 		if (ret < 0) {
@@ -973,6 +917,134 @@ int gnutls_handshake(int cd, GNUTLS_STATE state)
 			return ret;
 		}
 
+		/* FIXME: request - and get - a client certificate */
+		return 0;
+	}
+}
+
+/* This function sends the final handshake packets and initializes connection */
+static int _gnutls_send_handshake_final( int cd, GNUTLS_STATE state, int init) {
+int ret=0;
+
+		/* Send the CHANGE CIPHER SPEC PACKET */
+		ret = _gnutls_send_change_cipher_spec(cd, state);
+		if (ret < 0) {
+			ERR("send ChangeCipherSpec", ret);
+			gnutls_clearHashDataBuffer( state);
+			return ret;
+		}
+
+		/* Initialize the connection state (start encryption) - in case of client */
+		if (init == TRUE) {
+			ret = _gnutls_connection_state_init(state);
+			if (ret<0) {
+				gnutls_assert();
+				gnutls_clearHashDataBuffer( state);
+				return ret;
+			}
+		}
+		/* send the finished message */
+
+		ret = _gnutls_send_finished(cd, state);
+		if (ret < 0) {
+			ERR("send Finished", ret);
+			gnutls_clearHashDataBuffer( state);
+			return ret;
+		}
+		return ret;
+}
+
+/* This function receives the final handshake packets */
+static int _gnutls_recv_handshake_final( int cd, GNUTLS_STATE state, int init) {
+int ret=0;
+
+		ret =
+		    gnutls_recv_int(cd, state, GNUTLS_CHANGE_CIPHER_SPEC,
+				    NULL, 0, 0);
+		if (ret < 0) {
+			ERR("recv ChangeCipherSpec", ret);
+			gnutls_clearHashDataBuffer( state);
+			return ret;
+		}
+
+		/* Initialize the connection state (start encryption) - in case of server */
+		if (init == TRUE) {
+			ret = _gnutls_connection_state_init(state);
+			if (ret<0) {
+				gnutls_assert();
+				gnutls_clearHashDataBuffer( state);
+				return ret;
+			}
+		}
+
+		ret = _gnutls_recv_finished(cd, state);
+		if (ret < 0) {
+			ERR("recv finished", ret);
+			gnutls_clearHashDataBuffer( state);
+			return ret;
+		}
+		return ret;
+}
+
+/* in this function we finish the handshake procedure
+ * This should happen only if we trust the peer. (check certificate)
+ */
+int gnutls_handshake_finish(int cd, GNUTLS_STATE state)
+{
+	int ret=0;
+
+	if (state->security_parameters.entity == GNUTLS_CLIENT) {
+
+		/* receive the server key exchange */
+		if (state->gnutls_internals.resumed==RESUME_FALSE) /* if we are not resuming */
+		ret = _gnutls_recv_server_kx_message(cd, state);
+		if (ret < 0) {
+			ERR("recv server kx message", ret);
+			gnutls_clearHashDataBuffer( state);
+			return ret;
+		}
+
+		/* FIXME: receive certificate request */
+
+		/* receive the server hello done */
+		if (state->gnutls_internals.resumed==RESUME_FALSE) /* if we are not resuming */
+		ret =
+		    _gnutls_recv_handshake(cd, state, NULL, NULL,
+					   GNUTLS_SERVER_HELLO_DONE);
+		if (ret < 0) {
+			ERR("recv server hello done", ret);
+			gnutls_clearHashDataBuffer( state);
+			return ret;
+		}
+
+		/* send our certificate - if any */
+		if (state->gnutls_internals.resumed==RESUME_FALSE) /* if we are not resuming */
+		ret = _gnutls_send_client_certificate(cd, state);
+		if (ret < 0) {
+			ERR("send client certificate", ret);
+			gnutls_clearHashDataBuffer( state);
+			return ret;
+		}
+
+		if (state->gnutls_internals.resumed==RESUME_FALSE) /* if we are not resuming */
+		ret = _gnutls_send_client_kx_message(cd, state);
+		if (ret < 0) {
+			ERR("send client kx", ret);
+			gnutls_clearHashDataBuffer( state);
+			return ret;
+		}
+
+		/* send client certificate verify */
+		if (state->gnutls_internals.resumed==RESUME_FALSE) /* if we are not resuming */
+		ret = _gnutls_send_client_certificate_verify( cd, state);
+		if (ret < 0) {
+			ERR("send client certificate verify", ret);
+			gnutls_clearHashDataBuffer( state);
+			return ret;
+		}
+
+	} else { /* SERVER SIDE */
+
 		/* send the server hello done */
 		ret =
 		    _gnutls_send_handshake(cd, state, NULL, 0,
@@ -992,43 +1064,44 @@ int gnutls_handshake(int cd, GNUTLS_STATE state)
 			return ret;
 		}
 
-		ret =
-		    gnutls_recv_int(cd, state, GNUTLS_CHANGE_CIPHER_SPEC,
-				    NULL, 0);
-		if (ret < 0) {
-			ERR("recv ChangeCipherSpec", ret);
-			return ret;
-		}
-
-		/* Initialize the connection state (start encryption) */
-		ret = _gnutls_connection_state_init(state);
-		if (ret<0) {
-			gnutls_clearHashDataBuffer( state);
-			return ret;
-		}
-
-		ret = _gnutls_recv_finished(cd, state);
-		if (ret < 0) {
-			ERR("recv finished", ret);
-			gnutls_clearHashDataBuffer( state);
-			return ret;
-		}
-
-		ret = _gnutls_send_change_cipher_spec(cd, state);
-		if (ret < 0) {
-			ERR("send ChangeCipherSpec", ret);
-			gnutls_clearHashDataBuffer( state);
-			return ret;
-		}
-
-		ret = _gnutls_send_finished(cd, state);
-		if (ret < 0) {
-			ERR("send finished", ret);
-			gnutls_clearHashDataBuffer( state);
-			return ret;
-		}
-
 	}
+
+	/* send and recv the change cipher spec and finished messages */
+	if ((state->gnutls_internals.resumed==RESUME_TRUE && state->security_parameters.entity == GNUTLS_CLIENT) 
+	|| (state->gnutls_internals.resumed==RESUME_FALSE && state->security_parameters.entity == GNUTLS_SERVER)) { 
+		/* if we are a client resuming - or we are a server not resuming */
+
+		ret = _gnutls_recv_handshake_final( cd, state, TRUE);
+		if (ret < 0) {
+			gnutls_assert();
+			gnutls_clearHashDataBuffer( state);
+			return ret;
+		}
+
+		ret = _gnutls_send_handshake_final( cd, state, FALSE);
+		if (ret < 0) {
+			gnutls_assert();
+			gnutls_clearHashDataBuffer( state);
+			return ret;
+		}
+	} else { /* if we are a client not resuming - or we are a server resuming */
+
+		ret = _gnutls_send_handshake_final( cd, state, TRUE);
+		if (ret < 0) {
+			gnutls_assert();
+			gnutls_clearHashDataBuffer( state);
+			return ret;
+		}
+
+		ret = _gnutls_recv_handshake_final( cd, state, FALSE);
+		if (ret < 0) {
+			gnutls_assert();
+			gnutls_clearHashDataBuffer( state);
+			return ret;
+		}
+	}
+
+
 	/* clear handshake buffer */
 	gnutls_clearHashDataBuffer( state);
 	return ret;
