@@ -2,11 +2,13 @@
 
 #define GNUTLS_INT_H
 
-//#define HANDSHAKE_DEBUG
-//#define HARD_DEBUG
-//#define READ_DEBUG
-//#define WRITE_DEBUG
+/*
+#define HANDSHAKE_DEBUG
+#define HARD_DEBUG
+#define READ_DEBUG
+#define WRITE_DEBUG
 #define DEBUG
+*/
 
 #define MAX32 4294967295
 #define MAX24 16777215
@@ -47,7 +49,7 @@ typedef unsigned char opaque;
 
 
 enum ChangeCipherSpecType { GNUTLS_TYPE_CHANGE_CIPHER_SPEC=1 };
-enum AlertLevel { GNUTLS_WARNING, GNUTLS_FATAL };
+enum AlertLevel { GNUTLS_WARNING=1, GNUTLS_FATAL };
 enum AlertDescription { GNUTLS_CLOSE_NOTIFY, GNUTLS_UNEXPECTED_MESSAGE=10, GNUTLS_BAD_RECORD_MAC=20,
 			GNUTLS_DECRYPTION_FAILED, GNUTLS_RECORD_OVERFLOW,  GNUTLS_DECOMPRESSION_FAILURE=30,
 			GNUTLS_HANDSHAKE_FAILURE=40, GNUTLS_BAD_CERTIFICATE=42, GNUTLS_UNSUPPORTED_CERTIFICATE,
@@ -83,13 +85,12 @@ typedef struct {
 
 /* STATE */
 enum ConnectionEnd { GNUTLS_SERVER, GNUTLS_CLIENT };
-enum BulkCipherAlgorithm { GNUTLS_NULL, GNUTLS_ARCFOUR=1, GNUTLS_3DES = 4, GNUTLS_RIJNDAEL };
+enum BulkCipherAlgorithm { GNUTLS_NULL_CIPHER, GNUTLS_ARCFOUR=1, GNUTLS_3DES = 4, GNUTLS_RIJNDAEL };
 enum KXAlgorithm { GNUTLS_KX_RSA, GNUTLS_KX_DHE_DSS, GNUTLS_KX_DHE_RSA, GNUTLS_KX_DH_DSS, GNUTLS_KX_DH_RSA, GNUTLS_KX_ANON_DH };
 enum KeyExchangeAlgorithm { GNUTLS_RSA, GNUTLS_DIFFIE_HELLMAN };
 enum CipherType { CIPHER_STREAM, CIPHER_BLOCK };
-enum IsExportable { EXPORTABLE_TRUE, EXPORTABLE_FALSE };
-enum MACAlgorithm { GNUTLS_MAC_NULL, GNUTLS_MAC_MD5, GNUTLS_MAC_SHA };
-enum CompressionMethod { GNUTLS_COMPRESSION_NULL, GNUTLS_ZLIB=224 };
+enum MACAlgorithm { GNUTLS_NULL_MAC, GNUTLS_MAC_MD5, GNUTLS_MAC_SHA };
+enum CompressionMethod { GNUTLS_NULL_COMPRESSION, GNUTLS_ZLIB=224 };
 
 enum ValidSession { VALID_TRUE, VALID_FALSE };
 enum ResumableSession { RESUME_TRUE, RESUME_FALSE };
@@ -101,7 +102,6 @@ typedef enum ResumableSession ResumableSession;
 typedef enum ConnectionEnd ConnectionEnd;
 typedef enum BulkCipherAlgorithm BulkCipherAlgorithm;
 typedef enum CipherType CipherType;
-typedef enum IsExportable IsExportable;
 typedef enum MACAlgorithm MACAlgorithm;
 typedef enum CompressionMethod CompressionMethod;
 
@@ -112,16 +112,17 @@ typedef struct {
 	ConnectionEnd entity;
 	BulkCipherAlgorithm bulk_cipher_algorithm;
 	CipherType cipher_type;
+	MACAlgorithm mac_algorithm;
+	CompressionMethod compression_algorithm;
 	uint8 IV_size;
 	uint8 key_size;
 	uint8 key_material_length;
-	IsExportable is_exportable;
-	MACAlgorithm mac_algorithm;
 	uint8 hash_size;
-	CompressionMethod compression_algorithm;
 	opaque master_secret[48];
 	opaque client_random[32];
 	opaque server_random[32];
+	opaque session_id[32];
+	uint8 session_id_size;
 } SecurityParameters;
 
 typedef struct {
@@ -163,35 +164,35 @@ typedef struct {
 typedef struct {
 	int* algorithm_priority;
 	int algorithms;
-} BulkCipherAlgorithm_Priority;
+} GNUTLS_Priority;
 
-typedef struct {
-	int* algorithm_priority;
-	int algorithms;
-} MACAlgorithm_Priority;
-
-typedef struct {
-	int* algorithm_priority;
-	int algorithms;
-} KXAlgorithm_Priority;
-
-typedef struct {
-	int* algorithm_priority;
-	int algorithms;
-} CompressionMethod_Priority;
+#define BulkCipherAlgorithm_Priority GNUTLS_Priority
+#define MACAlgorithm_Priority GNUTLS_Priority
+#define KXAlgorithm_Priority GNUTLS_Priority
+#define CompressionMethod_Priority GNUTLS_Priority
 
 typedef struct {
 	char*			buffer;
 	uint32			bufferSize;
-	char*			hash_buffer; /* used in SSL3 */
-	uint32			hash_bufferSize; /* used in SSL3 */
-	char*			buffer_handshake;
+	char*			hash_buffer; /* used to keep all handshake messages */
+	uint32			hash_bufferSize;
+	char*			buffer_handshake; /* this is a buffer that holds the current handshake message */
 	uint32			bufferSize_handshake;
-	ResumableSession	resumable; /* TRUE or FALSE */
-	ValidSession		valid_connection; /* true or FALSE */
+	ResumableSession	resumable; /* TRUE or FALSE - if we can resume that session */
+	ValidSession		valid_connection; /* true or FALSE - if this session is valid */
 	AlertDescription	last_alert; /* last alert received */
+	/* this is the ciphersuite we are going to use */
 	GNUTLS_CipherSuite	current_cipher_suite;
+	/* this is the compression method we are going to use */
 	CompressionMethod	compression_method;
+	/* priorities */
+	BulkCipherAlgorithm_Priority	BulkCipherAlgorithmPriority;
+	MACAlgorithm_Priority		MACAlgorithmPriority;
+	KXAlgorithm_Priority		KXAlgorithmPriority;
+	CompressionMethod_Priority	CompressionMethodPriority;
+	/* resumed session */
+	ResumableSession	resumed; /* TRUE or FALSE - if we are resuming a session */
+	SecurityParameters  resumed_security_parameters;
 	/* For DH KX */
 	MPI				KEY;
 	MPI				client_Y;
@@ -200,10 +201,6 @@ typedef struct {
 	MPI				dh_secret;
 	int				certificate_requested; /* non zero if client certificate was requested */
 	int				certificate_verify_needed; /* non zero if we should expect for certificate verify */
-	BulkCipherAlgorithm_Priority	BulkCipherAlgorithmPriority;
-	MACAlgorithm_Priority		MACAlgorithmPriority;
-	KXAlgorithm_Priority		KXAlgorithmPriority;
-	CompressionMethod_Priority	CompressionMethodPriority;
 } GNUTLS_INTERNALS;
 
 typedef struct {
@@ -221,16 +218,13 @@ enum ContentType { GNUTLS_CHANGE_CIPHER_SPEC=20, GNUTLS_ALERT, GNUTLS_HANDSHAKE,
 		GNUTLS_APPLICATION_DATA };
 typedef enum ContentType ContentType;
 
-#define GNUTLS_DEFAULT_VERSION_MAJOR 3
-#define GNUTLS_DEFAULT_VERSION_MINOR 1
-
 typedef struct {
 	uint8 major;
 	uint8 minor;
 } ProtocolVersion;
 
 typedef struct {
-	uint8	type;
+	uint8		type;
 	ProtocolVersion	version;
 	uint16		length;
 	opaque*		fragment;
@@ -259,12 +253,12 @@ typedef struct {
 } GNUTLS_GenericStreamCipher;
 
 typedef struct {
-	uint8		type;
-	ProtocolVersion		version;
+	uint8			type;
+	ProtocolVersion	version;
 	uint16			length;
 	void*			fragment; /* points GenericStreamCipher
-					   * or GenericBlockCipher
-					   */
+					   		 * or GenericBlockCipher
+							 */
 } GNUTLSCiphertext;
 
 
@@ -284,7 +278,7 @@ typedef struct {
 
 
 typedef struct {
-	ProtocolVersion		client_version;
+	ProtocolVersion	client_version;
 	GNUTLS_random		random;
 	opaque*			session_id;
 	GNUTLS_CipherSuite*	cipher_suites;
@@ -292,7 +286,7 @@ typedef struct {
 } GNUTLS_ClientHello;
 
 typedef struct {
-	ProtocolVersion		server_version;
+	ProtocolVersion	server_version;
 	GNUTLS_random		random;
 	opaque*			session_id;
 	GNUTLS_CipherSuite	cipher_suite;
@@ -308,8 +302,8 @@ svoid *gnutls_PRF( opaque * secret, int secret_size, uint8 * label,
 void gnutls_set_current_version(GNUTLS_STATE state, GNUTLS_Version version);
 GNUTLS_Version gnutls_get_current_version(GNUTLS_STATE state);
 int _gnutls_set_keys(GNUTLS_STATE state);
-ssize_t gnutls_send_int(int cd, GNUTLS_STATE state, ContentType type, void* data, size_t sizeofdata);
-ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char* data, size_t sizeofdata);
+ssize_t gnutls_send_int(int cd, GNUTLS_STATE state, ContentType type, void* data, size_t sizeofdata, int flags);
+ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char* data, size_t sizeofdata, int flags);
 int _gnutls_send_change_cipher_spec(int cd, GNUTLS_STATE state);
 int _gnutls_version_cmp(GNUTLS_Version ver1, GNUTLS_Version ver2);
 #define _gnutls_version_ssl3(x) _gnutls_version_cmp(x, GNUTLS_SSL3)
