@@ -39,11 +39,9 @@ int main()
     int sd, ret;
     struct sockaddr_in sa_serv;
     struct sockaddr_in sa_cli;
-    int client_len, i;
+    int client_len;
     char topbuf[512];
     GNUTLS_STATE state;
-    char *str;
-    char buf[4096];
     char buffer[MAX_BUF];
 
 
@@ -63,8 +61,8 @@ int main()
     client_len = sizeof(sa_cli);
     for (;;) {
 	gnutls_init(&state, GNUTLS_SERVER);
-	gnutls_set_cipher_priority( state, 1, GNUTLS_3DES);
-	gnutls_set_compression_priority( state, 2, GNUTLS_COMPRESSION_NULL, GNUTLS_ZLIB);
+	gnutls_set_cipher_priority( state, 3, GNUTLS_RIJNDAEL, GNUTLS_3DES, GNUTLS_ARCFOUR);
+	gnutls_set_compression_priority( state, 2, GNUTLS_ZLIB, GNUTLS_NULL_COMPRESSION);
 	gnutls_set_kx_priority( state, 1, GNUTLS_KX_ANON_DH);
 	gnutls_set_mac_priority( state, 2, GNUTLS_MAC_SHA, GNUTLS_MAC_MD5);
 	sd = accept(listen_sd, (SA *) & sa_cli, &client_len);
@@ -86,8 +84,8 @@ int main()
 	}
 	fprintf(stderr, "Handshake was completed\n");
 	fprintf(stderr, "Acting as echo server...\n");
-	ret =
-	    gnutls_send(sd, state, "hello client",
+/*	ret =
+	    gnutls_write(sd, state, "hello client",
 			sizeof("hello client"));
 	if (ret < 0) {
 	    close(sd);
@@ -95,21 +93,22 @@ int main()
 	    gnutls_perror(ret);
 	    continue;
 	}
+*/
 	for (;;) {
 	    bzero( buffer, MAX_BUF);
-	    ret = gnutls_recv(sd, state, buffer, MAX_BUF);
+	    ret = gnutls_read(sd, state, buffer, MAX_BUF);
 	    if (gnutls_is_fatal_error(ret) == 1) {
 		if (ret == GNUTLS_E_CLOSURE_ALERT_RECEIVED) {
 		    fprintf(stderr,
 			    "\nPeer has closed the GNUTLS connection\n");
 		    break;
 		} else {
-		    fprintf(stderr, "\nReceived corrupted data. Closing the connection.\n");
+		    fprintf(stderr, "\nReceived corrupted data(%d). Closing the connection.\n", ret);
 		    break;
 		}
 
 	    }
-	    gnutls_send(sd, state, buffer, strlen(buffer));
+	    gnutls_write(sd, state, buffer, strlen(buffer));
 	}
 	fprintf(stderr, "\n");
 	gnutls_close(sd, state);
